"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var glob = require("glob");
var amqp = require("amqplib");
var appRoot = require("app-root-path");
var FunctionRegistry = /** @class */ (function () {
    function FunctionRegistry() {
        this.respondersDir = appRoot.path + '/dist/responders/*.js';
        this.consumersDir = appRoot.path + '/dist/consumers/*.js';
        this.globalConsumersDir = appRoot.path + '/dist/globalconsumers/*.js';
    }
    FunctionRegistry.prototype.init = function (rmqConfig) {
        var _this = this;
        return new Promise(function (res) {
            amqp.connect(rmqConfig.URL).then(function (connectedCon) {
                FunctionRegistry.CONN = connectedCon;
                FunctionRegistry.CONN.createChannel().then(function (ch) {
                    _this.initResponder(ch, rmqConfig).then(function () {
                        _this.initConsumer(ch, rmqConfig).then(function () {
                            res();
                        });
                    });
                });
            });
        });
    };
    FunctionRegistry.prototype.initResponder = function (ch, config) {
        var _this = this;
        return new Promise(function (res) {
            var instance;
            glob(_this.respondersDir, function (er, files) {
                files.forEach(function (file) {
                    // Split file path
                    var stringArray = file.split('/');
                    // Get the class name
                    var className = stringArray[stringArray.length - 1];
                    className = className.replace('.js', '');
                    instance = require(file)[className];
                    var temp = new instance();
                    var ListenTopicName = config.APP_NAME + '.' + temp.handleTopic;
                    ch.assertQueue(ListenTopicName, {
                        durable: config.RESPONDER_QUEUE_DURABLE,
                        autoDelete: config.RESPONDER_QUEUE_AUTODEL,
                        exclusive: config.RESPONDER_QUEUE_EXCLUSIVE,
                    });
                    ch.prefetch(1);
                    ch.consume(ListenTopicName, function reply(msg) {
                        var incomingData = JSON.parse(msg.content.toString());
                        temp
                            .executeWithResult(incomingData)
                            .then(function (response) {
                            ch.sendToQueue(msg.properties.replyTo, new Buffer(response.toString()), {
                                correlationId: msg.properties.correlationId,
                            });
                            ch.ack(msg);
                        })
                            .catch(function (err) {
                            console.log(err);
                            ch.nack(msg);
                        });
                    });
                    console.log(' [x] Responders registered for event : ' + ListenTopicName);
                });
                res();
            });
        });
    };
    FunctionRegistry.prototype.initConsumer = function (ch, config) {
        var _this = this;
        return new Promise(function (res) {
            var instance;
            glob(_this.consumersDir, function (er, files) {
                files.forEach(function (file) {
                    var stringArray = file.split('/');
                    var className = stringArray[stringArray.length - 1];
                    className = className.replace('.js', '');
                    instance = require(file)[className];
                    var temp = new instance();
                    var ListenTopicName = config.APP_NAME + '.' + temp.eventTopic;
                    ch.assertExchange(config.APP_NAME, 'direct', {
                        durable: config.CONSUMER_EXCHANGE_DURABLE,
                        autoDelete: config.CONSUMER_EXCHANGE_AUTODEL,
                    });
                    ch.assertQueue(ListenTopicName, {
                        durable: config.CONSUMER_QUEUE_DURABLE,
                        autoDelete: config.CONSUMER_QUEUE_AUTODEL,
                        exclusive: config.CONSUMER_QUEUE_EXCLUSIVE,
                    }).then(function (q) {
                        ch.bindQueue(q.queue, config.APP_NAME, temp.eventTopic);
                        ch.consume(q.queue, function reply(msg) {
                            var incomingData = JSON.parse(msg.content.toString());
                            var cmsg = msg;
                            temp
                                .handleEvent(incomingData)
                                .then(function () {
                                console.log('Consumed');
                                // check for timeout here and error out
                                ch.ack(cmsg);
                            })
                                .catch(function (err) {
                                console.log(err);
                                ch.nack(cmsg);
                            });
                        }, { noAck: false });
                    });
                    console.log(' [x] Consumer registered for event : ' + ListenTopicName);
                });
                res();
            });
        });
    };
    FunctionRegistry.prototype.initGlobalConsumer = function (ch, config) {
        var _this = this;
        return new Promise(function (res) {
            var instance;
            var appName = config.APP_NAME;
            var globalExchangeName = config.GLOBAL_EXCHANGE_NAME;
            if (!globalExchangeName || globalExchangeName.trim() === '') {
                throw new Error('globalExchangeName property is missing but is required for RMQBusJS.');
            }
            glob(_this.globalConsumersDir, function (er, files) {
                files.forEach(function (file) {
                    var stringArray = file.split('/');
                    var className = stringArray[stringArray.length - 1];
                    className = className.replace('.js', '');
                    instance = require(file)[className];
                    var temp = new instance();
                    var ListenTopicName = appName + '.' + temp.eventTopic;
                    ch.assertExchange(globalExchangeName, 'direct', {
                        durable: config.CONSUMER_EXCHANGE_DURABLE,
                        autoDelete: config.CONSUMER_EXCHANGE_AUTODEL,
                    });
                    ch.assertQueue(ListenTopicName, {
                        durable: config.CONSUMER_QUEUE_DURABLE,
                        autoDelete: config.CONSUMER_QUEUE_AUTODEL,
                        exclusive: config.CONSUMER_QUEUE_EXCLUSIVE,
                    }).then(function (q) {
                        ch.bindQueue(q.queue, globalExchangeName, temp.eventTopic);
                        ch.consume(q.queue, function reply(msg) {
                            var incomingData = JSON.parse(msg.content.toString());
                            var cmsg = msg;
                            temp
                                .handleEvent(incomingData)
                                .then(function () {
                                console.log('Consumed');
                                // check for timeout here and error out
                                ch.ack(cmsg);
                            })
                                .catch(function (err) {
                                console.log(err);
                                ch.nack(cmsg);
                            });
                        }, { noAck: false });
                    });
                    console.log(' [x] Global Consumer registered for event : ' + ListenTopicName);
                });
                res();
            });
        });
    };
    FunctionRegistry.prototype.getFunctionPaths = function () {
        return { responder: this.respondersDir, consumer: this.consumersDir };
    };
    return FunctionRegistry;
}());
exports.default = FunctionRegistry;
